/* @(#)scdb.c	14.1.1.4 (ESO-DMD) 07/28/00 16:42:10 */
/*===========================================================================
  Copyright (C) 1995 European Southern Observatory (ESO)
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 675 Massachusetts Ave, Cambridge, 
  MA 02139, USA.
 
  Correspondence concerning ESO-MIDAS should be addressed as follows:
	Internet e-mail: midas@eso.org
	Postal address: European Southern Observatory
			Data Management Division 
			Karl-Schwarzschild-Strasse 2
			D 85748 Garching bei Muenchen 
			GERMANY
===========================================================================*/

/*+++++++++++++++++++++++++++ SC interface module SCDB +++++++++++++++++++++++
.LANGUAGE C
.IDENTIFICATION Module SCDB
.AUTHOR         K. Banse 		ESO - Garching
.KEYWORDS       standard interfaces, descriptors
.ENVIRONMENT    VMS and UNIX
.COMMENTS	holds xwdscdir, xSCDWRi, SCDDEL, SCDGETC

.VERSION  [1.00] 920211:  pulled out from scd.c
.VERSION  [1.10] 930907: long int -> int !!!

 000726				last modification

----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <midas_def.h>
#include <fileexts.h>


#define DISK_REC    512


static char   dtype[4];

static int    dblock, dindx, dunit;

static struct FCB_STRUCT  *fcbp;

static struct FCT_STRUCT  *fctpntr;


/*

*/

int xwdscdir(imno,descr,bytelem,nval)

/*++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE
hdnale lower level descr directory stuff common to all descr tytpes
.ALGORITHM
.RETURNS
return status  ( 0 = o.k. )
--------------------------------------------------*/

int  imno	/* IN : no. of data frame */;
char  *descr	/* IN : descriptor name */;
int  bytelem  	/* IN : CHAR*noelm  */;
int  nval	/* IN : no. of data values to write */;

{
int   status;




fctpntr = FCT.ENTRIES + imno;
fcbp = fctpntr->FZP;

dunit = 0;			/* not used yet */
status = MID_ZDSCDIR(imno,'a',descr,dtype,&bytelem,
                     &nval,&dunit,&dblock,&dindx); 	/* add descr */
fctpntr->KAUX[2] = 1;

return status;
}
	
/*

*/

xSCDWRH(imno,descr,values,felem,nval)

/*++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE
Write help text of a descriptor
.ALGORITHM
The help text is added and linked into the descr. entry. This routine
must be called directly after the descr itself has been written,
otherwise it is NOT safe!
.RETURNS
return status  ( 0 = o.k. )
.REMARKS
if 'felem' = -1, descriptor values are appended to existing ones
--------------------------------------------------*/

int   imno	/* IN : no. of data frame */;
char   *descr   /* IN : descriptor name (max. 15 char.) */;
char   *values  /* IN : buffer with descriptor data */;
int felem	/* IN : position of 1st descriptor value to be accessed */;
int nval	/* IN : no. of data values to write */;

{
int   status, noelem, newl;
register int  nr;



/* check help text ignore flag */

if ((KEYALL.ORIGIN != -1) && (KIWORDS[OFF_MONPAR+13] == 1))
   return ERR_NORMAL;


dtype[0] = 'H';
status = xwdscdir(imno,descr,1,nval);

if (status == ERR_NORMAL)
   {
   if (nval > 0) 
      (void) MID_WDSCRC(fctpntr->IOCHAN,dblock,dindx,values,1,nval);
   }
else
   MID_E2(5,imno,descr,status,1);

return (status);
}
	
/*

*/

xSCDWRC(imno,descr,noelm,values,felem,nval,unit)

/*++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE
Write a descriptor and its data
.ALGORITHM
A new descriptor is written at the end of the existing 
descriptor area or exisitng descriptor data is modified
and extended, if necessary.
.RETURNS
return status  ( 0 = o.k. )
.REMARKS
if 'felem' = -1, descriptor values are appended to existing ones
--------------------------------------------------*/

int   imno	/* IN : no. of data frame */;
char   *descr   /* IN : descriptor name (max. 15 char.) */;
int noelm  	/* IN : CHAR*noelm  */;
char   *values  /* IN : buffer with descriptor data */;
int felem	/* IN : position of 1st descriptor value to be accessed */;
int nval	/* IN : no. of data values to write */;
int   *unit	/* IN : unit-pointer */;

{
int   status, noelem, newl;
register int  nr;



dtype[0] = 'C';	
status = xwdscdir(imno,descr,noelm,nval);

if (status == ERR_NORMAL)
   {
   newl = nval * noelm;		/* char array -> flat string */
   for (nr=0; nr<newl; nr++)
      {
      if (values[nr] == '\0')		/* we don't have enough data */
         {
         char  *xpntr, *zpntr;
         int   kk;
         register int mr;

         kk = nr;
         xpntr = malloc((unsigned int)newl);
         if (xpntr == (char *) 0)
            {
            xpntr = values;
            newl = kk;			/* all what we can do ...  */
            }
         else
            {
            zpntr = xpntr;
            (void) strcpy(zpntr,values);	/* first, copy the data */
            zpntr += kk;
            for (mr=kk; mr<newl; mr++)	/* then, pad with blanks */
               *zpntr++ = ' ';
            }

         (void) MID_WDSCRC(fctpntr->IOCHAN,dblock,dindx,xpntr,1,newl);
         free(xpntr);		/* free allocated memory again */
         return status;
         }
      }

   (void) MID_WDSCRC(fctpntr->IOCHAN,dblock,dindx,values,1,newl);
   }
else
   MID_E2(5,imno,descr,status,1);

return (status);
}

/*

*/

xSCDWRD(imno,descr,values,felem,nval,unit)

/*++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE
Write a descriptor and its data
.ALGORITHM
  A new descriptor is written at the end of the existing 
  descriptor area or existing descriptor data is modified
  and extended, if necessary.
.RETURNS
return status  ( 0 = o.k. )
.REMARKS
if 'felem' = -1, descriptor values are appended to existing ones
--------------------------------------------------*/

int   imno	/* IN : no. of data frame */;
char   *descr   /* IN : descriptor name (max. 15 char.) */;
double *values  /* IN : buffer with descriptor data */;
int felem	/* IN : position of 1st descriptor value to be accessed */;
int  nval	/* IN : no. of data values to write */;
int   *unit	/* IN : unit-pointer */;

{
int   n, newlen, status, *ipntr;


	

dtype[0] = 'D';	
status = xwdscdir(imno,descr,DD_SIZE,nval);

if (status == ERR_NORMAL) 
   {		     /* use MID_WDSCRI to write double precision descr... */
   n = DD_SIZE/II_SIZE;
   newlen = n * nval;
   ipntr = (int *) values;
   (void) MID_WDSCRI(fctpntr->IOCHAN,dblock,dindx,ipntr,1,newlen);
   }
else
   MID_E2(5,imno,descr,status,1);
return (status);
}

/*

*/

xSCDWRI(imno,descr,values,felem,nval,unit)

/*++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE
Write a descriptor and its data
.ALGORITHM
  A new descriptor is written at the end of the existing 
  descriptor area or exisitng descriptor data is modified
  and extended, if necessary.
.RETURNS
return status  ( 0 = o.k. )
.REMARKS
if 'felem' = -1, descriptor values are appended to existing ones
--------------------------------------------------*/

int   imno	/* IN : no. of data frame */;
char   *descr   /* IN : descriptor name (max. 15 char.) */;
int *values  /* IN : buffer with descriptor data */;
int felem	/* IN : position of 1st descriptor value to be accessed */;
int  nval	/* IN : no. of data values to write */;
int   *unit	/* IN : unit-pointer */;

{
int   status;




dtype[0] = 'I';
status = xwdscdir(imno,descr,II_SIZE,nval);
	
if (status == ERR_NORMAL)
   (void) MID_WDSCRI(fctpntr->IOCHAN,dblock,dindx,values,1,nval);
else
   MID_E2(5,imno,descr,status,1);
return (status);
}

/*

*/

xSCDWRR(imno,descr,values,felem,nval,unit)

/*++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE
Write a descriptor and its data
.ALGORITHM
  A new descriptor is written at the end of the existing 
  descriptor area or exisitng descriptor data is modified
  and extended, if necessary.
.RETURNS
return status  ( 0 = o.k. )
.REMARKS
if 'felem' = -1, descriptor values are appended to existing ones
--------------------------------------------------*/

int   imno	/* IN : no. of data frame */;
char   *descr   /* IN : descriptor name (max. 15 char.) */;
float *values  /* IN : buffer with descriptor data */;
int felem	/* IN : position of 1st descriptor value to be accessed */;
int  nval	/* IN : no. of data values to write */;
int   *unit	/* IN : unit-pointer */;

{
int   status;




dtype[0] = 'R';
status = xwdscdir(imno,descr,RR_SIZE,nval);
	

if (status == ERR_NORMAL)
      (void) MID_WDSCRR(fctpntr->IOCHAN,dblock,dindx,values,1,nval);
else
    MID_E2(5,imno,descr,status,1);
return (status);
}

/*

*/

SCDDEL(imno,descr)

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE
  delete a descriptor 
.ALGORITHM
  the descriptor directory is scanned for the specified name and if found,
  that name is marked as deleted 
  If descriptor = ' ', reinitialize descriptor area.
.RETURNS
return status  ( 0 = o.k. )
-----------------------------------------------------------------------*/

int imno;     		/* IN : no. of data frame */
char	*descr;		/* IN : descriptor name */

{
int   status, bytelem, noelem, dblock, dindx, dunit;

char   dtype[4];

struct FCB_STRUCT  *fcbp;

struct FCT_STRUCT  *fctpntr;




if ( (imno < 0) || (imno >= FCT.MAXENT) )
   {
   status = ERR_INPINV;
   goto end_of_it;
   }

fctpntr = FCT.ENTRIES + imno;
fcbp = fctpntr->FZP;


/*  if descriptor name is '*', reinitialize descriptor area  */
 
if (*descr == '*')
   status = MID_INITDS(fcbp,fctpntr->IOCHAN);   /* do the rest via MID_INITDS */
 
else
   {				 /*  otherwise delete single descriptor  */
   dtype[0] = ' ';
   if (fcbp->DSCFLAG == 'Z')		/* not needed for new descr. format */
      {
      status = MID_ZDSCDIR(imno,'F',descr,dtype,
                          &bytelem,&noelem,&dunit,&dblock,&dindx);
      if (status == ERR_NORMAL) 
         status = MID_ZDSCDIR(imno,'D',descr,dtype,
                             &bytelem,&noelem,&dunit,&dblock,&dindx);
      }
   else
      {
      status = MID_DSCDIR(imno,'F',descr,dtype,
                          &bytelem,&noelem,&dunit,&dblock,&dindx);
      if (status == ERR_NORMAL) 
         status = MID_DSCDIR(imno,'D',descr,dtype,
                             &bytelem,&noelem,&dunit,&dblock,&dindx);
      if (status != ERR_NORMAL)	goto end_of_it;

      dtype[0] = 'H';			/* also look for Help descr. */
      status = MID_DSCDIR(imno,'F',descr,dtype,
                          &bytelem,&noelem,&dunit,&dblock,&dindx);
      if (status == ERR_NORMAL) 
         status = MID_DSCDIR(imno,'D',descr,dtype,
                             &bytelem,&noelem,&dunit,&dblock,&dindx);
      else
         status = ERR_NORMAL;		/* that's not an error */
      }
   }

fctpntr->KAUX[2] = 1;		/* set descriptor-modified flag */
 

end_of_it:				/*  test for info/warnings again  */
if (status != ERR_NORMAL) 
   MID_E2(4,imno,descr,status,1);
 
return (status);
}

/*

*/

SCDGETC(imno,descr,felem,maxvals,actvals,values)

/*++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE
get data from character descriptor and terminate with Null (no trailing blanks)
.ALGORITHM
use SCDRDC
.RETURNS
return status  ( 0 = o.k. )
--------------------------------------------------*/

int   imno	  /* IN : no. of data frame */;
char    *descr    /* IN : descriptor name */;
int felem	  /* IN : position of 1st element to be accessed */;
int maxvals  /* IN : max. no. of characters to be returned */;
int *actvals /* OUT: actual no. of characters returned */;
char    *values   /* OUT: array for descriptor data */;

{
int   status, mynull, myuni;
register int  nr;

register char    mychar;



mynull = -1;
status = SCDRDC(imno,descr,1,felem,maxvals,actvals,values,&myuni,&mynull);



if (status == ERR_NORMAL)
   {				/* get rid of trailing blanks/nulls */
   maxvals = (*actvals) - 1;
   for (nr=maxvals; nr>=0; nr--)
      {
      mychar = values[nr];
      if ((mychar != ' ') && (mychar != '\0'))
         {
         *actvals = ++nr;
         values[nr] = '\0';		/* add null terminator  */
         return (status);
         }
      }
   }

values[0] = '\0';
*actvals = 0;
return (status);
}

