/* @(#)ihaprhd.c	14.1.1.1 (ES0-DMD) 09/16/99 09:21:25 */
/*===========================================================================
  Copyright (C) 1995 European Southern Observatory (ESO)
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
  MA 02139, USA.
 
  Corresponding concerning ESO-MIDAS should be addressed as follows:
	Internet e-mail: midas@eso.org
	Postal address: European Southern Observatory
			Data Management Division 
			Karl-Schwarzschild-Strasse 2
			D 85748 Garching bei Muenchen 
			GERMANY
===========================================================================*/

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT  (c)  1993   European Southern Observatory
.IDENT      ihaprhd.c
.LAUGUAGE   C
.AUTHOR     P.Grosbol   ESO/IPG
.KEYWORDS   IHAP header, decode
.COMMENT    main IHAP header is decoded including comments
.VERSION    1.0  1988-Nov-28 : Creation,   PJG 
.VERSION    1.1  1989-Oct-17 : Include MMF-format + ext. dir., PJG 
.VERSION    1.2  1989-Oct-26 : Extend Identifier to 30 char., PJG 
.VERSION    1.3  1990-Jan-30 : Write IDENT even when missing, PJG 
.VERSION    1.4  1990-Feb-04 : Change call-seq. of cv-routine, PJG 
.VERSION    1.45 1990-Mar-08 : Change CUNIT desc. to C*1, PJG 
.VERSION    1.5  1990-Jun-20 : Include special PDS parameters, PJG 
.VERSION    2.0  1991-Feb-17 : Change include-file and structure, PJG 
.VERSION    2.1  1993-Feb-25 : Correct reading of comments, PJG 
.VERSION    2.2  1993-Nov-25 : Update to new ST + prototypes, PJG 
---------------------------------------------------------------------*/
#include   <math.h>
#include   <fitsfmt.h>
#include   <fitsdef.h>
#include   <midas_def.h>

#define    HSIZE         1024      /* size of IHAP header in bytes   */

typedef struct {                   /* comment format in ext-dir.     */
		 INT2       no;    /* no. of bytes in string         */
                 BYTE     c[1];    /* character string               */
               } XDIR;

typedef struct {                   /* format of comment block        */
		 INT2       no;    /* no. of bytes in string         */
		 INT2     hour;    /* time of comment - hour         */
		 UINT1     min;    /* time of comment - min          */
		 UINT1     sec;    /* time of comment - sec          */
		 INT2    dummy;    /* no. of bytes in string         */
                 BYTE     c[1];    /* character string               */
               } CBLK;

#ifdef __STDC__
int ihaprhd(int *pmfd,BFDEF *bfdef,int *psize,char fmt,int popt)
#else
int ihaprhd(pmfd,bfdef,psize,fmt,popt)
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE       decode IHAP header directory
.RETURN        type of IHAP file i.e. 'jcont' in IHAP header
---------------------------------------------------------------------*/
int        *pmfd;              /* OUT: MIDAS file descriptor         */
BFDEF      *bfdef;             /* OUT: FITS/IHAP header parm's       */
int        *psize;             /* OUT: size of data matrix (bytes)   */
char       fmt;                /* IN:  MIDAS file format No/Orig/Fp  */
int        popt;               /* IN:  print leve                    */
#endif
{
  char     c, *pc, *pbuf, cmt[80];
  short    *ps, *psb;
  int      mfd, dtype, ftype, size, njd, i, n, m, npix[2], unit[4];
  int      cntr, code, jidnt, jseq, jrdin, jyear, jday;
  int      jmeth, jcont, jfrmt, jusr1, jusr2, ifhgc, iflwc, pds;
  int      felem, ii;
  float    drasc, decln, dairm, dcrat, dsttm, dentm, ssttm;
  float    exptime, dusr1, dusr2, ahgc, alwc, flval;
  float    user[5], lhcuts[6];
  double   start[2], step[2], o_time[8], o_pos[4];
  XDIR     *xdir;
  CBLK     *cblk;
  union { char   c[172]; int   i[43]; float  f[43]; }   rbuf;

  n = dread(&pbuf,HSIZE);                   /* get IHAP header block */
  pds = pbuf[9] == 8;                       /* check if its from PDS */

  pc = pbuf + 54;                             /* IHAP identifier     */
  for (i=0; i<30 && *pc; i++)                 /* max. 30 bytes       */
    bfdef->ident[i] = *pc++;
  bfdef->ident[i] = '\0'; njd = i;            /* NULL term. string   */

  pc = pbuf + 4; n = 0;                       /* flt_val : size      */
  for (i=0; i<4; i++) rbuf.c[n++] = *pc++;
  pc = pbuf + 14;                             /* sky coordinates     */
  for (i=0; i<12; i++) rbuf.c[n++] = *pc++;
  pc = pbuf + 38;                             /* obs. time           */
  for (i=0; i<16; i++) rbuf.c[n++] = *pc++;
  pc = pbuf + 102;                            /* world coordinates   */
  for (i=0; i<24; i++) rbuf.c[n++] = *pc++;
  pc = pbuf + 130;                            /* user reals          */
  for (i=0; i<8; i++) rbuf.c[n++] = *pc++;
  pc = pbuf + 160;                            /* high cut value      */
  for (i=0; i<4; i++) rbuf.c[n++] = *pc++;
  pc = pbuf + 166;                            /* low cut value       */
  for (i=0; i<4; i++) rbuf.c[n++] = *pc++;
  if (pds) {
     pc = pbuf + 388;
     for (i=0; i<44; i++) rbuf.c[n++] = *pc++;
     pc = pbuf + 444;
     for (i=0; i<8; i++) rbuf.c[n++] = *pc++;
     pc = pbuf + 464;
     for (i=0; i<48; i++) rbuf.c[n++] = *pc++;
  }
  else for (i=0; i<100; i++) rbuf.c[n++] = '\0';

  psb = (short *) pbuf;
  i = (pds) ? 256 : 128;                      /* convert main header */
  cvi2(psb,i,0);
  cntr = *psb++; code = *psb;
  switch (code) {                         /* check IHAP format code  */
         case  1 : cvr4(rbuf.f,43,0);              /* convert reals  */
                   break;
         case 20 : cvi4(rbuf.i,43,0);              /* convert int4's */
                   for (i=0; i<43; i++) { 
                       ii = rbuf.i[i]; rbuf.f[i] = ii;
                   }
                   for (i=1; i<8; i++) rbuf.f[i] = 0.0; 
                   break;
         default : return -1;
  }
  flval = rbuf.f[0];
  psb += 2;                               /* adjust for IDX offset   */
  ps = psb + 1;
  jidnt = *ps++; jseq = *ps++; jrdin = *ps;
  ps = psb + 14;
  jyear = *ps++; jday = *ps;
  ps = psb + 39; jmeth = *ps++; jcont = *ps;
  ps = psb + 42; jfrmt = *ps;
  bfdef->bitpix = (jfrmt==3) ? -32 : 16;
  ps = psb + 60; jusr1 = *ps++; jusr2 = *ps;
  ps = psb + 76; ifhgc = *ps;
  ps = psb + 79; iflwc = *ps;

  bfdef->naxis = (jmeth==1) ? 1 : 2;        /* get coordinates       */
  bfdef->data[0].naxis = floor((rbuf.f[9]-rbuf.f[8])/rbuf.f[12] + 1.5);
  bfdef->data[0].crval = rbuf.f[8];
  bfdef->data[0].cdelt = rbuf.f[12];
  bfdef->data[1].naxis = 1;
  if (bfdef->naxis==2) {
     bfdef->data[1].naxis = floor((rbuf.f[11]-rbuf.f[10])/rbuf.f[13] + 1.5);
     bfdef->data[1].crval = rbuf.f[10];
     bfdef->data[1].cdelt = rbuf.f[13];
  }
  bfdef->sflag = (jmeth==3);
  for (i=0; i<bfdef->naxis; i++) {
    npix[i] = bfdef->data[i].naxis;
    start[i] = bfdef->data[i].crval;
    step[i] = bfdef->data[i].cdelt;
  }
                                           /* observation data etc. */
  drasc = (jrdin) ? rbuf.f[1]/240.0 : 0.0;
  decln = (jrdin) ? rbuf.f[2]/3600.0 : 0.0;
  dairm = rbuf.f[3];
  dcrat = rbuf.f[4];
  dsttm = rbuf.f[5]; dentm = rbuf.f[6];
  ssttm = rbuf.f[7];
  exptime = dentm - dsttm;
  if (exptime<0) exptime += 86400.0;
                                           /* IHAP variables        */
  dusr1 = rbuf.f[14]; dusr2 = rbuf.f[15];
  ahgc = (ifhgc) ? rbuf.f[16] : 0.0;
  alwc = (iflwc) ? rbuf.f[17] : 0.0;

/*+++++++++++++++++++   create MIDAS file   ++++++++++++++++++++++++*/
  if (fmt=='O' && bfdef->bitpix==16) {     /* I2 data format        */
     dtype = D_I2_FORMAT; ftype = 1;
  }
  else {                                   /* R4 data format, image */
     dtype = D_R4_FORMAT; ftype = 1;
  }
  unit[0] = 0;
  pc = newfn('I',"");
  if (fmt!='N') {
    size = bfdef->data[0].naxis * bfdef->data[1].naxis;
    SCFCRE(pc,dtype,F_O_MODE,ftype,size,&mfd);
    *pmfd = mfd;
    SCDWRI(mfd,"NAXIS",&(bfdef->naxis),1,1,unit);
    SCDWRI(mfd,"NPIX",npix,1,bfdef->naxis,unit);
    SCDWRD(mfd,"START",start,1,bfdef->naxis,unit);
    SCDWRD(mfd,"STEP",step,1,bfdef->naxis,unit);
    if (njd) SCDWRC(mfd,"IDENT",1,bfdef->ident,1,njd,unit);
       else SCDWRC(mfd,"IDENT",1," ",1,1,unit);
/*    SCDWRC(mfd,"CUNIT",16,"",1,1,unit); */
/*    SCDWRC(mfd,"CUNIT",16,"",2,1,unit); */
/*    SCDWRC(mfd,"CUNIT",16,"",3,1,unit); */
    SCDWRC(mfd,"CUNIT",1,"",1,16,unit);
    SCDWRC(mfd,"CUNIT",1,"",17,16,unit);
    SCDWRC(mfd,"CUNIT",1,"",33,16,unit);
    lhcuts[0] = alwc; lhcuts[1] = ahgc;
    lhcuts[2] = 0.0; lhcuts[3] = 0.0;
    lhcuts[4] = 0.0; lhcuts[5] = 0.0;
    SCDWRR(mfd,"LHCUTS",lhcuts,1,6,unit);
    user[0] = (float) jusr1; user[1] = (float) jusr2;
    user[2] = dusr1; user[3] = dusr2; user[4] = (float) jseq;
    SCDWRR(mfd,"IHAPUSER",user,1,5,unit);
    o_pos[0] = drasc; o_pos[1] = decln;
    SCDWRD(mfd,"O_POS",o_pos,1,2,unit);
    o_time[0] = dateymd(jyear,1,jday); o_time[1] = 0.0; o_time[2] = 0.0;
    o_time[3] = 0.0; o_time[4] = dsttm/3600.0; o_time[5] = ssttm/3600.0;
    o_time[6] = exptime; o_time[7] = 0.0;
    SCDWRD(mfd,"O_TIME",o_time,1,8,unit);
    SCDWRR(mfd,"O_AIRM",&dairm,1,1,unit);
    if (pds) {                          /* store special PDS parameters */
       SCDWRR(mfd,"PDS_PARM",&rbuf.f[18],1,5,unit);
       SCDWRR(mfd,"PDS_TRAN",&rbuf.f[23],1,6,unit);
       SCDWRR(mfd,"PDS_APER",&rbuf.f[29],1,2,unit);
       SCDWRR(mfd,"PDS_REFXY",&rbuf.f[31],1,12,unit);
       ps = psb + 213;
       for (i=0; i<6; i++) rbuf.i[i] = *ps++;
       SCDWRI(mfd,"PDS_IVAL",rbuf.i,1,6,unit);
    }
  }
  if (popt) {
     sprintf(cmt,"Image %-16.16s : %-16.16s , naxis: %2d, pixels: %8d, %5d",
	     pc,bfdef->ident,bfdef->naxis,
	     bfdef->data[0].naxis,bfdef->data[1].naxis);
     SCTPUT(cmt);
  }
  if (fmt=='N') return jcont;
                                       /* decode extended directory */
  felem = 1;
  pc = pbuf + 256; xdir = (XDIR *) pc; m = 0;
  while (xdir->no && m<256) {
     cvi2((short *) xdir,1,0);
     if ((n=xdir->no)==0) break;
     if (n<0 || 36<n) xdir->no = n = 36;
     n *= 2;
     for (i=0; i<n && ' '<=(c=xdir->c[i]) && c<='~'; i++) cmt[i] = c;
     cmt[i++] = '\n'; cmt[i] = '\0';
     SCDWRC(mfd,"O_COM",1,cmt,felem,i,unit);
     if (0<felem) felem = -1;
     if (1<popt) { cmt[--i] = '\0'; SCTPUT(cmt); }
     m +=2*xdir->no + 2;
     pc += 2*xdir->no + 2; xdir = (XDIR *) pc;
  }
                                       /* decode comment block      */
  pc = pbuf + 512; cblk = (CBLK *) pc; m = 0;
  while (cblk->no && m<512) {
     cvi2((short *) cblk,2,0);
     if ((n=cblk->no)==0) break;
     cvi2(&cblk->hour,2,0);
     if (cblk->hour<0 || 24<cblk->hour) break;
     if (n<0 || 36<n) cblk->no = n = 36;
     n *= 2;
     sprintf(cmt,"%02d:%02d>",cblk->hour,cblk->min);
     for (i=6; i<n && ' '<=(c=cblk->c[i-6]) && c<='~'; i++) cmt[i] = c;
     cmt[i++] = '\n'; cmt[i] = '\0';
     SCDWRC(mfd,"O_COM",1,cmt,felem,i,unit);
     if (0<felem) felem = -1;
     if (1<popt) { cmt[--i] = '\0'; SCTPUT(cmt); }
     m += 2*cblk->no + 8;
     pc += 2*cblk->no + 8; cblk = (CBLK *) pc;
  }

  if (cntr) {
     sprintf(cmt,"Warning: %d Info. blocks is NOT decoded!",cntr);
     SCTPUT(cmt);
   }

  size = bfdef->data[0].naxis * bfdef->data[1].naxis * (jfrmt+1);
  *psize = floor(2.0*flval+0.5);
  if (size != (*psize)) 
      SCTPUT("Warning: Truncation error in no. of pixels!");
  return jcont;
}
